@@ -7,6 +7,7 @@ from .widgets import (
     DateTimeEdit,
     Dialog,
     EmptyWidget,
+    FloatRangeSlider,
     FloatSlider,
     FloatSpinBox,
     Image,
@@ -18,6 +19,7 @@ from .widgets import (
     PushButton,
     RadioButton,
     RadioButtons,
+    RangeSlider,
     Select,
     Slider,
     SpinBox,
@@ -37,8 +39,10 @@ __all__ = [
     "DateTimeEdit",
     "Dialog",
     "EmptyWidget",
+    "FloatRangeSlider",
     "FloatSlider",
     "FloatSpinBox",
+    "get_text_width",
     "Image",
     "Label",
     "LineEdit",
@@ -48,12 +52,12 @@ __all__ = [
     "PushButton",
     "RadioButton",
     "RadioButtons",
+    "RangeSlider",
     "Select",
+    "show_file_dialog",
     "Slider",
     "SpinBox",
     "Table",
     "TextEdit",
     "TimeEdit",
-    "get_text_width",
-    "show_file_dialog",
 ]
