@@ -3,11 +3,14 @@ from __future__ import annotations
 
 import math
 import re
+import warnings
 from contextlib import contextmanager
 from functools import partial
-from typing import TYPE_CHECKING, Any, Iterable, Sequence
+from itertools import chain
+from typing import TYPE_CHECKING, Any, Iterable, Optional, Sequence
 
 import qtpy
+import superqt
 from qtpy import QtWidgets as QtW
 from qtpy.QtCore import QEvent, QObject, Qt, Signal
 from qtpy.QtGui import (
@@ -53,8 +56,10 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     _qwidget: QtW.QWidget
 
-    def __init__(self, qwidg: QtW.QWidget):
-        self._qwidget = qwidg()
+    def __init__(
+        self, qwidg: type[QtW.QWidget], parent: Optional[QtW.QWidget] = None, **kwargs
+    ):
+        self._qwidget = qwidg(parent=parent)
         self._qwidget.setObjectName(f"magicgui.{qwidg.__name__}")
         self._event_filter = EventFilter()
         self._qwidget.installEventFilter(self._event_filter)
@@ -84,13 +89,16 @@ class QBaseWidget(_protocols.WidgetProtocol):
     def _mgui_get_native_widget(self) -> QtW.QWidget:
         return self._qwidget
 
+    def _mgui_get_root_native_widget(self) -> QtW.QWidget:
+        return self._qwidget
+
     def _mgui_get_width(self) -> int:
         """Return the current width of the widget."""
         return self._qwidget.width()
 
     def _mgui_set_width(self, value: int) -> None:
         """Set the current width of the widget."""
-        self._qwidget.resize(value, self._qwidget.height())
+        self._qwidget.resize(int(value), self._qwidget.height())
 
     def _mgui_get_min_width(self) -> int:
         """Get the minimum allowable width of the widget."""
@@ -98,7 +106,7 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     def _mgui_set_min_width(self, value: int) -> None:
         """Set the minimum allowable width of the widget."""
-        self._qwidget.setMinimumWidth(value)
+        self._qwidget.setMinimumWidth(int(value))
         self._qwidget.resize(self._qwidget.sizeHint())
 
     def _mgui_get_max_width(self) -> int:
@@ -107,7 +115,7 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     def _mgui_set_max_width(self, value: int) -> None:
         """Set the maximum allowable width of the widget."""
-        self._qwidget.setMaximumWidth(value)
+        self._qwidget.setMaximumWidth(int(value))
         self._qwidget.resize(self._qwidget.sizeHint())
 
     def _mgui_get_height(self) -> int:
@@ -116,7 +124,7 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     def _mgui_set_height(self, value: int) -> None:
         """Set the current height of the widget."""
-        self._qwidget.resize(self._qwidget.width(), value)
+        self._qwidget.resize(self._qwidget.width(), int(value))
 
     def _mgui_get_min_height(self) -> int:
         """Get the minimum allowable height of the widget."""
@@ -124,7 +132,7 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     def _mgui_set_min_height(self, value: int) -> None:
         """Set the minimum allowable height of the widget."""
-        self._qwidget.setMinimumHeight(value)
+        self._qwidget.setMinimumHeight(int(value))
         self._qwidget.resize(self._qwidget.sizeHint())
 
     def _mgui_get_max_height(self) -> int:
@@ -133,13 +141,13 @@ class QBaseWidget(_protocols.WidgetProtocol):
 
     def _mgui_set_max_height(self, value: int) -> None:
         """Set the maximum allowable height of the widget."""
-        self._qwidget.setMaximumHeight(value)
+        self._qwidget.setMaximumHeight(int(value))
         self._qwidget.resize(self._qwidget.sizeHint())
 
     def _mgui_get_tooltip(self) -> str:
         return self._qwidget.toolTip()
 
-    def _mgui_set_tooltip(self, value: str | None) -> None:
+    def _mgui_set_tooltip(self, value: Optional[str]) -> None:
         self._qwidget.setToolTip(str(value) if value else None)
 
     def _mgui_bind_parent_change_callback(self, callback):
@@ -169,8 +177,10 @@ class QBaseWidget(_protocols.WidgetProtocol):
 class QBaseValueWidget(QBaseWidget, _protocols.ValueWidgetProtocol):
     """Implements get/set/bind_change."""
 
-    def __init__(self, qwidg: QtW.QWidget, getter: str, setter: str, onchange: str):
-        super().__init__(qwidg)
+    def __init__(
+        self, qwidg: QtW.QWidget, getter: str, setter: str, onchange: str, **kwargs
+    ):
+        super().__init__(qwidg, **kwargs)
         self._getter_name = getter
         self._setter_name = setter
         self._onchange_name = onchange
@@ -199,8 +209,8 @@ class QBaseValueWidget(QBaseWidget, _protocols.ValueWidgetProtocol):
 
 
 class EmptyWidget(QBaseWidget):
-    def __init__(self):
-        super().__init__(QtW.QWidget)
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QWidget, **kwargs)
 
     def _mgui_get_value(self) -> Any:
         raise NotImplementedError()
@@ -225,8 +235,8 @@ class QBaseStringWidget(QBaseValueWidget):
 class Label(QBaseStringWidget):
     _qwidget: QtW.QLabel
 
-    def __init__(self):
-        super().__init__(QtW.QLabel, "text", "setText", "")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QLabel, "text", "setText", "", **kwargs)
         self._qwidget.setSizePolicy(QtW.QSizePolicy.Fixed, QtW.QSizePolicy.Fixed)
 
     def _mgui_bind_change_callback(self, callback):
@@ -248,8 +258,8 @@ class _ResizeableLabel(QtW.QLabel):
 class Image(QBaseValueWidget):
     _qwidget: _ResizeableLabel
 
-    def __init__(self):
-        super().__init__(_ResizeableLabel, "text", "setText", "")
+    def __init__(self, **kwargs):
+        super().__init__(_ResizeableLabel, "text", "setText", "", **kwargs)
         self._qwidget.setSizePolicy(QtW.QSizePolicy.Ignored, QtW.QSizePolicy.Ignored)
         self._qwidget.resized.connect(self._rescale)
         self._pixmap: QPixmap = None
@@ -270,15 +280,15 @@ class Image(QBaseValueWidget):
 class LineEdit(QBaseStringWidget):
     _qwidget: QtW.QLineEdit
 
-    def __init__(self):
-        super().__init__(QtW.QLineEdit, "text", "setText", "textChanged")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QLineEdit, "text", "setText", "textChanged", **kwargs)
 
 
 class LiteralEvalLineEdit(QBaseStringWidget):
     _qwidget: QtW.QLineEdit
 
-    def __init__(self):
-        super().__init__(QtW.QLineEdit, "text", "setText", "textChanged")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QLineEdit, "text", "setText", "textChanged", **kwargs)
 
     def _post_get_hook(self, value):
         from ast import literal_eval
@@ -287,8 +297,10 @@ class LiteralEvalLineEdit(QBaseStringWidget):
 
 
 class TextEdit(QBaseStringWidget, _protocols.SupportsReadOnly):
-    def __init__(self):
-        super().__init__(QtW.QTextEdit, "toPlainText", "setText", "textChanged")
+    def __init__(self, **kwargs):
+        super().__init__(
+            QtW.QTextEdit, "toPlainText", "setText", "textChanged", **kwargs
+        )
 
     def _mgui_set_read_only(self, value: bool) -> None:
         self._qwidget.setReadOnly(value)
@@ -303,11 +315,11 @@ class TextEdit(QBaseStringWidget, _protocols.SupportsReadOnly):
 class QBaseRangedWidget(QBaseValueWidget, _protocols.RangedWidgetProtocol):
     """Provides min/max/step implementations."""
 
-    _qwidget: QtW.QDoubleSpinBox | QtW.QSpinBox | QtW.QSlider
+    _qwidget: QtW.QDoubleSpinBox | QtW.QSpinBox | QtW.QAbstractSlider
     _precision: float = 1
 
-    def __init__(self, qwidg):
-        super().__init__(qwidg, "value", "setValue", "valueChanged")
+    def __init__(self, qwidg, **kwargs):
+        super().__init__(qwidg, "value", "setValue", "valueChanged", **kwargs)
 
     def _mgui_get_min(self) -> float:
         """Get the minimum possible value."""
@@ -364,8 +376,8 @@ class QBaseRangedWidget(QBaseValueWidget, _protocols.RangedWidgetProtocol):
 class QBaseButtonWidget(QBaseValueWidget, _protocols.SupportsText):
     _qwidget: QtW.QCheckBox | QtW.QPushButton | QtW.QRadioButton | QtW.QToolButton
 
-    def __init__(self, qwidg):
-        super().__init__(qwidg, "isChecked", "setChecked", "toggled")
+    def __init__(self, qwidg, **kwargs):
+        super().__init__(qwidg, "isChecked", "setChecked", "toggled", **kwargs)
 
     def _mgui_set_text(self, value: str) -> None:
         """Set text."""
@@ -377,32 +389,34 @@ class QBaseButtonWidget(QBaseValueWidget, _protocols.SupportsText):
 
 
 class PushButton(QBaseButtonWidget):
-    def __init__(self):
+    def __init__(self, **kwargs):
         QBaseValueWidget.__init__(
-            self, QtW.QPushButton, "isChecked", "setChecked", "clicked"
+            self, QtW.QPushButton, "isChecked", "setChecked", "clicked", **kwargs
         )
+        # make enter/return "click" the button when focused.
+        self._qwidget.setAutoDefault(True)
 
 
 class CheckBox(QBaseButtonWidget):
-    def __init__(self):
-        super().__init__(QtW.QCheckBox)
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QCheckBox, **kwargs)
 
 
 class RadioButton(QBaseButtonWidget):
-    def __init__(self):
-        super().__init__(QtW.QRadioButton)
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QRadioButton, **kwargs)
 
 
 # class ToolButton(QBaseButtonWidget):
-#     def __init__(self):
-#         super().__init__(QtW.QToolButton)
+#     def __init__(self,**kwargs):
+#         super().__init__(QtW.QToolButton, **kwargs)
 
 
 class Container(
     QBaseWidget, _protocols.ContainerProtocol, _protocols.SupportsOrientation
 ):
-    def __init__(self, layout="vertical", scrollable: bool = False):
-        QBaseWidget.__init__(self, QtW.QWidget)
+    def __init__(self, layout="vertical", scrollable: bool = False, **kwargs):
+        QBaseWidget.__init__(self, QtW.QWidget, **kwargs)
         if layout == "horizontal":
             self._layout: QtW.QBoxLayout = QtW.QHBoxLayout()
         else:
@@ -428,7 +442,11 @@ class Container(
     def _is_scrollable(self) -> bool:
         return isinstance(self._qwidget, QtW.QScrollArea)
 
+    def _mgui_get_root_native_widget(self):
+        return self._qwidget
+
     def _mgui_get_native_widget(self):
+        # Return widget with the layout set
         return self._qwidget.widget() if self._is_scrollable else self._qwidget
 
     def _mgui_get_visible(self):
@@ -470,8 +488,8 @@ class Container(
 
 
 class MainWindow(Container):
-    def __init__(self, layout="vertical", scrollable: bool = False):
-        super().__init__(layout=layout, scrollable=scrollable)
+    def __init__(self, layout="vertical", scrollable: bool = False, **kwargs):
+        super().__init__(layout=layout, scrollable=scrollable, **kwargs)
         self._main_window = QtW.QMainWindow()
         self._menus: dict[str, QtW.QMenu] = {}
         if scrollable:
@@ -503,8 +521,8 @@ class MainWindow(Container):
 
 
 class SpinBox(QBaseRangedWidget):
-    def __init__(self):
-        super().__init__(QtW.QSpinBox)
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QSpinBox, **kwargs)
 
     def _mgui_set_value(self, value) -> None:
         super()._mgui_set_value(int(value))
@@ -514,8 +532,8 @@ class SpinBox(QBaseRangedWidget):
 
 
 class FloatSpinBox(QBaseRangedWidget):
-    def __init__(self):
-        super().__init__(QtW.QDoubleSpinBox)
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QDoubleSpinBox, **kwargs)
 
     def _mgui_set_value(self, value) -> None:
         super()._mgui_set_value(float(value))
@@ -530,11 +548,17 @@ class FloatSpinBox(QBaseRangedWidget):
 class _Slider(QBaseRangedWidget, _protocols.SupportsOrientation):
     _qwidget: QtW.QSlider
 
-    def __init__(self, qwidg=QtW.QSlider, **kwargs):
-        super().__init__(qwidg)
+    def __init__(
+        self,
+        qwidg=QtW.QSlider,
+        readout: bool = True,
+        orientation: str = "horizontal",
+        **kwargs,
+    ):
+        super().__init__(qwidg, **kwargs)
         self._mgui_set_orientation("horizontal")
-        self._mgui_set_readout_visibility(kwargs.get("readout", True))
-        self._mgui_set_orientation(kwargs.get("orientation", "horizontal"))
+        self._mgui_set_readout_visibility(readout)
+        self._mgui_set_orientation(orientation)
 
     def _mgui_set_orientation(self, value) -> Any:
         """Get current value of the widget."""
@@ -571,7 +595,7 @@ class Slider(_Slider):
     def __init__(self, qwidg=QtW.QSlider, **kwargs):
         self._container = QtW.QWidget()
         self._readout_widget = self._readout()
-        super().__init__(qwidg)
+        super().__init__(qwidg, **kwargs)
 
         self._readout_widget.setButtonSymbols(self._readout_widget.NoButtons)
         self._readout_widget.setStyleSheet("background:transparent; border: 0;")
@@ -621,20 +645,23 @@ class Slider(_Slider):
     def _mgui_set_min(self, value: float):
         """Set the minimum possible value."""
         super()._mgui_set_min(value)
-        self._readout_widget.setMinimum(value)
+        self._readout_widget.setMinimum(self._pre_set_hook(value))
 
     def _mgui_set_max(self, value: float):
         """Set the maximum possible value."""
         super()._mgui_set_max(value)
-        self._readout_widget.setMaximum(value)
+        self._readout_widget.setMaximum(self._pre_set_hook(value))
 
     def _mgui_set_step(self, value: float):
         """Set the step size."""
         super()._mgui_set_step(value)
-        self._readout_widget.setSingleStep(value)
+        self._readout_widget.setSingleStep(self._pre_set_hook(value))
 
     def _mgui_get_adaptive_step(self) -> bool:
-        return self._readout_widget.stepType() == QtW.QAbstractSpinBox.AdaptiveStep
+        return (
+            self._readout_widget.stepType()
+            == QtW.QAbstractSpinBox.AdaptiveDecimalStepType
+        )
 
     def _mgui_set_adaptive_step(self, value: bool):
         self._readout_widget.setStepType(
@@ -692,11 +719,51 @@ class FloatSlider(Slider):
         self._qwidget.valueChanged.connect(_converted_value)
 
 
+class RangeSlider(_Slider):
+    _qwidget: superqt.QLabeledRangeSlider
+
+    def __init__(self, **kwargs):
+        super().__init__(superqt.QLabeledRangeSlider, **kwargs)
+        if hasattr(self._qwidget, "applyMacStylePatch"):
+            # >= magicgui v0.5.2
+            self._qwidget.applyMacStylePatch()
+
+    def _mgui_set_readout_visibility(self, value: bool):
+        method = "show" if value else "hide"
+        try:
+            for label in chain(
+                self._qwidget._handle_labels,
+                [self._qwidget._min_label, self._qwidget._max_label],
+            ):
+                getattr(label, method)()
+        except AttributeError as e:
+            warnings.warn(str(e))
+
+    def _mgui_set_adaptive_step(self, value: bool):
+        pass
+
+    def _mgui_get_adaptive_step(self) -> bool:
+        return False
+
+    def _pre_set_hook(self, value):
+        return value
+
+
+class FloatRangeSlider(RangeSlider):
+    _qwidget: superqt.QLabeledDoubleRangeSlider
+
+    def __init__(self, **kwargs):
+        _Slider.__init__(self, superqt.QLabeledDoubleRangeSlider, **kwargs)
+        if hasattr(self._qwidget, "applyMacStylePatch"):
+            # >= magicgui v0.5.2
+            self._qwidget.applyMacStylePatch()
+
+
 class ProgressBar(_Slider):
     _qwidget: QtW.QProgressBar
 
     def __init__(self, **kwargs):
-        super().__init__(QtW.QProgressBar)
+        super().__init__(QtW.QProgressBar, **kwargs)
 
     def _mgui_get_step(self) -> float:
         """Get the step size."""
@@ -718,8 +785,8 @@ class ProgressBar(_Slider):
 class ComboBox(QBaseValueWidget, _protocols.CategoricalWidgetProtocol):
     _qwidget: QtW.QComboBox
 
-    def __init__(self):
-        super().__init__(QtW.QComboBox, "isChecked", "setCurrentIndex", "")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QComboBox, "isChecked", "setCurrentIndex", "", **kwargs)
         self._qwidget.currentIndexChanged.connect(self._emit_data)
 
     def _emit_data(self, index: int):
@@ -807,8 +874,8 @@ class ComboBox(QBaseValueWidget, _protocols.CategoricalWidgetProtocol):
 class Select(QBaseValueWidget, _protocols.CategoricalWidgetProtocol):
     _qwidget: QtW.QListWidget
 
-    def __init__(self):
-        super().__init__(QtW.QListWidget, "isChecked", "setCurrentIndex", "")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QListWidget, "isChecked", "setCurrentIndex", "", **kwargs)
         self._qwidget.itemSelectionChanged.connect(self._emit_data)
         self._qwidget.setSelectionMode(QtW.QAbstractItemView.ExtendedSelection)
 
@@ -905,8 +972,8 @@ class RadioButtons(
 ):
     _qwidget: QtW.QGroupBox
 
-    def __init__(self):
-        super().__init__(QtW.QGroupBox, "", "", "")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QGroupBox, "", "", "", **kwargs)
         self._btn_group = QtW.QButtonGroup(self._qwidget)
         self._mgui_set_orientation("vertical")
         self._btn_group.buttonToggled.connect(self._emit_data)
@@ -1004,8 +1071,10 @@ class RadioButtons(
 
 
 class DateTimeEdit(QBaseValueWidget):
-    def __init__(self):
-        super().__init__(QtW.QDateTimeEdit, "", "setDateTime", "dateTimeChanged")
+    def __init__(self, **kwargs):
+        super().__init__(
+            QtW.QDateTimeEdit, "", "setDateTime", "dateTimeChanged", **kwargs
+        )
 
     def _mgui_get_value(self):
         try:
@@ -1015,8 +1084,8 @@ class DateTimeEdit(QBaseValueWidget):
 
 
 class DateEdit(QBaseValueWidget):
-    def __init__(self):
-        super().__init__(QtW.QDateEdit, "", "setDate", "dateChanged")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QDateEdit, "", "setDate", "dateChanged", **kwargs)
 
     def _mgui_get_value(self):
         try:
@@ -1026,8 +1095,8 @@ class DateEdit(QBaseValueWidget):
 
 
 class TimeEdit(QBaseValueWidget):
-    def __init__(self):
-        super().__init__(QtW.QTimeEdit, "", "setTime", "timeChanged")
+    def __init__(self, **kwargs):
+        super().__init__(QtW.QTimeEdit, "", "setTime", "timeChanged", **kwargs)
 
     def _mgui_get_value(self):
         try:
@@ -1037,8 +1106,8 @@ class TimeEdit(QBaseValueWidget):
 
 
 class Dialog(QBaseWidget, _protocols.ContainerProtocol):
-    def __init__(self, layout="vertical", **k):
-        QBaseWidget.__init__(self, QtW.QDialog)
+    def __init__(self, layout="vertical", scrollable: bool = False, **kwargs):
+        QBaseWidget.__init__(self, QtW.QDialog, **kwargs)
         if layout == "horizontal":
             self._layout: QtW.QBoxLayout = QtW.QHBoxLayout()
         else:
@@ -1114,7 +1183,7 @@ def get_text_width(text: str) -> int:
     if _might_be_rich_text(text):
         doc = QTextDocument()
         doc.setHtml(text)
-        return doc.size().width()
+        return math.ceil(doc.size().width())
     else:
         fm = QFontMetrics(QFont("", 0))
         return fm.boundingRect(text).width() + 5
@@ -1221,8 +1290,8 @@ class Table(QBaseWidget, _protocols.TableWidgetProtocol):
     _EDITABLE = QtW.QTableWidget.EditKeyPressed | QtW.QTableWidget.DoubleClicked
     _READ_ONLY = QtW.QTableWidget.NoEditTriggers
 
-    def __init__(self):
-        super().__init__(_QTableExtended)
+    def __init__(self, **kwargs):
+        super().__init__(_QTableExtended, **kwargs)
         header = self._qwidget.horizontalHeader()
         # avoid strange AttributeError on CI
         if hasattr(header, "setSectionResizeMode"):
