@@ -7,10 +7,22 @@ import pathlib
 import types
 import warnings
 from collections import defaultdict
+from contextlib import contextmanager
 from enum import EnumMeta
-from typing import Any, Callable, DefaultDict, ForwardRef, Type, TypeVar, cast, overload
+from typing import (
+    Any,
+    Callable,
+    DefaultDict,
+    ForwardRef,
+    Iterator,
+    Optional,
+    Type,
+    TypeVar,
+    cast,
+    overload,
+)
 
-from typing_extensions import Literal
+from typing_extensions import Literal, get_origin
 
 from magicgui import widgets
 from magicgui.types import (
@@ -83,14 +95,23 @@ def match_type(tw: TypeWrapper) -> WidgetTuple | None:
     if _type in (types.FunctionType,):
         return widgets.FunctionGui, {"function": tw.default}  # type: ignore
 
-    # sequence of paths
     if tw.shape in tw.SHAPE.SEQUENCE_LIKE:
         if _is_subclass(tw.type_, pathlib.Path):
+            # sequence of paths
             return widgets.FileEdit, {"mode": "rm"}
         elif tw.shape == tw.SHAPE.LIST:
             return widgets.ListEdit, {}
         elif tw.shape == tw.SHAPE.TUPLE:
             return widgets.TupleEdit, {}
+
+    if tw.shape == tw.SHAPE.SET:
+        if len(_type.__args__) > 0:
+            arg = _type.__args__[0]
+            if get_origin(arg) is Literal:
+                return widgets.Select, {"choices": arg.__args__}
+
+    if get_origin(_type) is Literal:
+        return widgets.ComboBox, {"choices": _type.__args__}
     return None
 
 
@@ -134,6 +155,7 @@ def pick_widget_type(
     annotation: type[Any] | None = None,
     options: WidgetOptions | None = None,
     is_result: bool = False,
+    raise_on_unknown: bool = True,
 ) -> WidgetTuple:
     """Pick the appropriate widget type for ``value`` with ``annotation``."""
     if is_result and annotation is inspect.Parameter.empty:
@@ -187,6 +209,11 @@ def pick_widget_type(
         _cls, opts = _widget_type
         return _cls, {**options, **opts}  # type: ignore
 
+    if raise_on_unknown:
+        raise ValueError(
+            f"No widget found for type {_type} and annotation {annotation}"
+        )
+
     return widgets.EmptyWidget, {"visible": False}
 
 
@@ -195,6 +222,7 @@ def get_widget_class(
     annotation: type[Any] | None = None,
     options: WidgetOptions | None = None,
     is_result: bool = False,
+    raise_on_unknown: bool = True,
 ) -> tuple[WidgetClass, WidgetOptions]:
     """Return a WidgetClass appropriate for the given parameters.
 
@@ -210,6 +238,8 @@ def get_widget_class(
     is_result : bool, optional
         Identifies whether the returned widget should be tailored to
         an input or to an output.
+    raise_on_unknown : bool, optional
+        Raise exception if no widget is found for the given type, by default True
 
     Returns
     -------
@@ -219,7 +249,9 @@ def get_widget_class(
     """
     _options = cast(WidgetOptions, options)
 
-    widget_type, _options = pick_widget_type(value, annotation, _options, is_result)
+    widget_type, _options = pick_widget_type(
+        value, annotation, _options, is_result, raise_on_unknown
+    )
 
     if isinstance(widget_type, str):
         widget_class: WidgetClass = _import_class(widget_type)
@@ -360,6 +392,63 @@ def register_type(
     return _deco if type_ is None else _deco(type_)
 
 
+@contextmanager
+def type_registered(
+    type_: _T,
+    *,
+    widget_type: WidgetRef | None = None,
+    return_callback: ReturnCallback | None = None,
+    **options,
+) -> Iterator[None]:
+    """Context manager that temporarily registers a widget type for a given `type_`.
+
+    When the context is exited, the previous widget type associations for `type_` is
+    restored.
+
+    Parameters
+    ----------
+    type_ : _T
+        The type for which a widget class or return callback will be provided.
+    widget_type : Optional[WidgetRef]
+        A widget class from the current backend that should be used whenever ``type_``
+        is used as the type annotation for an argument in a decorated function,
+        by default None
+    return_callback: Optional[callable]
+        If provided, whenever ``type_`` is declared as the return type of a decorated
+        function, ``return_callback(widget, value, return_type)`` will be called
+        whenever the decorated function is called... where ``widget`` is the Widget
+        instance, and ``value`` is the return value of the decorated function.
+    **options
+        key value pairs where the keys are valid `WidgetOptions`
+    """
+    tw = TypeWrapper(type_)
+    tw.resolve()
+    _type_ = tw.outer_type_
+
+    # check if return_callback is already registered
+    rc_was_present = return_callback in _RETURN_CALLBACKS.get(_type_, [])
+    # store any previous widget_type and options for this type
+    prev_type_def: Optional[WidgetTuple] = _TYPE_DEFS.get(_type_, None)
+    _type_ = register_type(
+        type_, widget_type=widget_type, return_callback=return_callback, **options
+    )
+    new_type_def: Optional[WidgetTuple] = _TYPE_DEFS.get(_type_, None)
+    try:
+        yield
+    finally:
+        # restore things to before the context
+        if return_callback is not None and not rc_was_present:
+            _RETURN_CALLBACKS[_type_].remove(return_callback)
+
+        if _TYPE_DEFS.get(_type_, None) is not new_type_def:
+            warnings.warn("Type definition changed during context", stacklevel=2)
+
+        if prev_type_def is not None:
+            _TYPE_DEFS[_type_] = prev_type_def
+        else:
+            _TYPE_DEFS.pop(_type_, None)
+
+
 def _type2callback(type_: type) -> list[ReturnCallback]:
     """Check if return callbacks have been registered for ``type_`` and return if so.
 
