@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import inspect
 from typing import TYPE_CHECKING, Any, Callable, MutableSequence, Sequence, overload
 
@@ -73,7 +74,9 @@ class ContainerWidget(Widget, _OrientationMixin, MutableSequence[Widget]):
         self._list: list[Widget] = []
         self._labels = labels
         self._layout = layout
-        kwargs["backend_kwargs"] = {"layout": layout, "scrollable": scrollable}
+        self._scrollable = scrollable
+        kwargs.setdefault("backend_kwargs", {})
+        kwargs["backend_kwargs"].update({"layout": layout, "scrollable": scrollable})
         super().__init__(**kwargs)
         self.extend(widgets)
         self.parent_changed.connect(self.reset_choices)
@@ -317,16 +320,24 @@ class ContainerWidget(Widget, _OrientationMixin, MutableSequence[Widget]):
         from pathlib import Path
 
         path = Path(path)
-        if not path.exists() and quiet:
-            return
-        for key, val in pickle.loads(path.read_bytes()).items():
-            val = pickle.loads(val)
-            if val == self.NO_VALUE:
-                continue
-            try:
-                getattr(self, key).value = val
-            except (ValueError, AttributeError):
-                pass
+        if not path.exists():
+            if quiet:
+                return
+            raise FileNotFoundError(f"Widget state file does not exist: {path}")
+
+        try:
+            data: dict = pickle.loads(path.read_bytes())
+        except Exception:
+            if quiet:
+                path.unlink(missing_ok=True)
+                return
+            raise
+
+        for key, val in data.items():
+            with contextlib.suppress(ValueError, AttributeError):
+                val = pickle.loads(val)
+                if val != self.NO_VALUE:
+                    getattr(self, key).value = val
 
 
 class MainWindowWidget(ContainerWidget):
