@@ -136,6 +136,7 @@ class FunctionGui(Container, Generic[_R]):
         param_options: dict[str, dict] | None = None,
         name: str = None,
         persist: bool = False,
+        raise_on_unknown=False,
         **kwargs,
     ):
         if not callable(function):
@@ -151,8 +152,11 @@ class FunctionGui(Container, Generic[_R]):
         elif not isinstance(param_options, dict):
             raise TypeError("'param_options' must be a dict of dicts")
 
-        sig = magic_signature(function, gui_options=param_options)
+        sig = magic_signature(
+            function, gui_options=param_options, raise_on_unknown=raise_on_unknown
+        )
         self.return_annotation = sig.return_annotation
+        self._tooltips = tooltips
         if tooltips:
             _inject_tooltips_from_docstrings(function.__doc__, sig)
 
@@ -164,7 +168,7 @@ class FunctionGui(Container, Generic[_R]):
         # access attributes (like `__name__` that only function objects have).
         # Mypy doesn't seem catch this at this point:
         # https://github.com/python/mypy/issues/9934
-        name = getattr(function, "__name__", None)
+        name = name or getattr(function, "__name__", None)
         if not name:
             if hasattr(function, "func"):  # partials:
                 f = getattr(function, "func", None)
@@ -184,6 +188,7 @@ class FunctionGui(Container, Generic[_R]):
         self._param_options = param_options
         self._result_name = ""
         self._call_count: int = 0
+        self._bound_instances: dict[int, FunctionGui] = {}
 
         # a deque of Progressbars to be created by (possibly nested) tqdm_mgui iterators
         self._tqdm_pbars: Deque[ProgressBar] = deque()
@@ -219,6 +224,7 @@ class FunctionGui(Container, Generic[_R]):
                 annotation=self._return_annotation,
                 gui_only=True,
                 is_result=True,
+                raise_on_unknown=raise_on_unknown,
             )
             self.append(self._result_widget)
 
@@ -371,10 +377,13 @@ class FunctionGui(Container, Generic[_R]):
             auto_call=self._auto_call,
             result_widget=bool(self._result_widget),
             app=None,
+            persist=self.persist,
+            visible=self.visible,
+            tooltips=self._tooltips,
+            scrollable=self._scrollable,
+            name=self.name,
         )
 
-    _bound_instances: dict[int, FunctionGui] = {}
-
     def __get__(self, obj, objtype=None) -> FunctionGui:
         """Provide descriptor protocol.
 
